import { RequestHandler, Router } from 'express';
import { Services } from '../../app';
import { Create, Read, Login } from '../../functions/user';
import { getId } from './middleware';

export function LoginHandler(services: Services): RequestHandler {
  const login = Login(services);

  return async (req, res, next) => {
    const user = await login(getId(req), req.body);
    user
      ? res.status(200).send(user)
      : next({ status: 404, message: 'bad user or password' });
  };
}

export function CreateHandler(services: Services): RequestHandler {
  const createUser = Create(services);

  return async (req, res, next) => {
    const user = await createUser(getId(req), req.body);
    user ? res.status(201).send(user) : next();
  };
}

export function ReadHandler(services: Services): RequestHandler {
  const readUser = Read(services);

  return async (req, res, next) => {
    const user = await readUser(getId(req), req.params.uuid);
    user
      ? res.status(200).send(user)
      : next({ status: 404, message: 'user not found' });
  };
}

export function getRoutes(services: Services) {
  const router = Router();

  router.post('/login', LoginHandler(services));
  router.get('/read/:uuid', ReadHandler(services));
  router.post('/create', CreateHandler(services));

  return router;
}
